package javax.swing.plaf.basic;

import java.awt.Container;
import javax.swing.text.Position;
import java.awt.Graphics;
import sun.swing.SwingUtilities2;
import javax.swing.text.html.ImageView;
import javax.swing.text.Element;
import java.awt.Color;
import java.awt.Font;
import javax.swing.text.html.StyleSheet;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.BadLocationException;
import javax.swing.text.AttributeSet;
import javax.swing.text.ParagraphView;
import javax.swing.text.html.HTML;
import javax.swing.text.StyleConstants;
import java.awt.Shape;
import java.awt.Rectangle;
import javax.swing.text.Document;
import java.io.StringReader;
import javax.swing.text.html.HTMLDocument;
import java.net.URL;
import javax.swing.text.View;
import javax.swing.JComponent;
import javax.swing.text.ViewFactory;

public class BasicHTML
{
    private static final String htmlDisable = "html.disable";
    public static final String propertyKey = "html";
    public static final String documentBaseKey = "html.base";
    private static BasicHTML.BasicEditorKit basicHTMLFactory;
    private static ViewFactory basicHTMLViewFactory;
    private static final String styleChanges = "p { margin-top: 0; margin-bottom: 0; margin-left: 0; margin-right: 0 }body { margin-top: 0; margin-bottom: 0; margin-left: 0; margin-right: 0 }";
    
    public BasicHTML() {
    }
    
    public static View createHTMLView(JComponent component, String s) {
        BasicHTML.BasicEditorKit factory = getFactory();
        Document defaultDocument = factory.createDefaultDocument(component.getFont(), component.getForeground());
        Object clientProperty = component.getClientProperty("html.base");
        if ((clientProperty instanceof URL)) {
            ((HTMLDocument)defaultDocument).setBase((URL)clientProperty);
        }
        StringReader in = new StringReader(s);
        try {
            factory.read(in, defaultDocument, 0);
        }
        catch (Throwable t) {}
        ViewFactory viewFactory = factory.getViewFactory();
        return new BasicHTML.Renderer(component, viewFactory, viewFactory.create(defaultDocument.getDefaultRootElement()));
    }
    
    public static int getHTMLBaseline(View view, int n, int n2) {
        if (n < 0 || n2 < 0) {
            throw new IllegalArgumentException("Width and height must be >= 0");
        }
        if ((view instanceof BasicHTML.Renderer)) {
            return getBaseline(view.getView(0), n, n2);
        }
        return -1;
    }
    
    static int getBaseline(JComponent component, int n, int n2, int n3, int n4) {
        View view = (View)component.getClientProperty("html");
        if (view != null) {
            int htmlBaseline = getHTMLBaseline(view, n3, n4);
            if (htmlBaseline < 0) {
                return htmlBaseline;
            }
            return n + htmlBaseline;
        }
        else {
            return n + n2;
        }
    }
    
    static int getBaseline(View view, int width, int height) {
        if (hasParagraph(view)) {
            view.setSize((float)width, (float)height);
            return getBaseline(view, new Rectangle(0, 0, width, height));
        }
        return -1;
    }
    
    private static int getBaseline(View view, Shape childAllocation) {
        if (view.getViewCount() == 0) {
            return -1;
        }
        AttributeSet attributes = view.getElement().getAttributes();
        Object attribute = null;
        if (attributes != null) {
            attribute = attributes.getAttribute(StyleConstants.NameAttribute);
        }
        int n = 0;
        if (attribute == HTML.Tag.HTML && view.getViewCount() > 1) {
            ++n;
        }
        childAllocation = view.getChildAllocation(n, childAllocation);
        if (childAllocation == null) {
            return -1;
        }
        View view2 = view.getView(n);
        if ((view instanceof ParagraphView)) {
            Rectangle bounds;
            if ((childAllocation instanceof Rectangle)) {
                bounds = (Rectangle)childAllocation;
            }
            else {
                bounds = childAllocation.getBounds();
            }
            return bounds.y + (int)((float)bounds.height * view2.getAlignment(1));
        }
        else {
            return getBaseline(view2, childAllocation);
        }
    }
    
    private static boolean hasParagraph(View view) {
        if ((view instanceof ParagraphView)) {
            return true;
        }
        if (view.getViewCount() == 0) {
            return false;
        }
        AttributeSet attributes = view.getElement().getAttributes();
        Object attribute = null;
        if (attributes != null) {
            attribute = attributes.getAttribute(StyleConstants.NameAttribute);
        }
        int n = 0;
        if (attribute == HTML.Tag.HTML && view.getViewCount() > 1) {
            n = 1;
        }
        return hasParagraph(view.getView(n));
    }
    
    public static boolean isHTMLString(String s) {
        return s != null && s.length() >= 6 && s.charAt(0) == '<' && s.charAt(5) == '>' && s.substring(1, 5).equalsIgnoreCase("html");
    }
    
    public static void updateRenderer(JComponent component, String s) {
        View htmlView = null;
        View view = (View)component.getClientProperty("html");
        if ((Boolean)component.getClientProperty("html.disable") != Boolean.TRUE && isHTMLString(s)) {
            htmlView = createHTMLView(component, s);
        }
        if (htmlView != view && view != null) {
            for (int i = 0; i < view.getViewCount(); ++i) {
                view.getView(i).setParent(null);
            }
        }
        component.putClientProperty("html", htmlView);
        String s2 = (String)component.getClientProperty("AccessibleName");
        String trim = null;
        if (s2 != null && view != null) {
            try {
                trim = view.getDocument().getText(0, view.getDocument().getLength()).trim();
            }
            catch (BadLocationException ex) {}
        }
        if (s2 == null || s2.equals(trim)) {
            String trim2 = null;
            if (htmlView != null) {
                try {
                    trim2 = htmlView.getDocument().getText(0, htmlView.getDocument().getLength()).trim();
                }
                catch (BadLocationException ex2) {}
            }
            component.putClientProperty("AccessibleName", trim2);
        }
    }
    
    static BasicHTML.BasicEditorKit getFactory() {
        if (BasicHTML.basicHTMLFactory == null) {
            BasicHTML.basicHTMLViewFactory = new BasicHTML.BasicHTMLViewFactory();
            BasicHTML.basicHTMLFactory = new BasicHTML.BasicEditorKit();
        }
        return BasicHTML.basicHTMLFactory;
    }
    
    static class BasicEditorKit extends HTMLEditorKit
    {
        private static StyleSheet defaultStyles;
        
        @Override
        public StyleSheet getStyleSheet() {
            if (BasicHTML.BasicEditorKit.defaultStyles == null) {
                BasicHTML.BasicEditorKit.defaultStyles = new StyleSheet();
                StringReader in = new StringReader("p { margin-top: 0; margin-bottom: 0; margin-left: 0; margin-right: 0 }body { margin-top: 0; margin-bottom: 0; margin-left: 0; margin-right: 0 }");
                try {
                    BasicHTML.BasicEditorKit.defaultStyles.loadRules(in, null);
                }
                catch (Throwable t) {}
                in.close();
                BasicHTML.BasicEditorKit.defaultStyles.addStyleSheet(super.getStyleSheet());
            }
            return BasicHTML.BasicEditorKit.defaultStyles;
        }
        
        public Document createDefaultDocument(Font font, Color color) {
            StyleSheet styleSheet = this.getStyleSheet();
            StyleSheet styleSheet2 = new StyleSheet();
            styleSheet2.addStyleSheet(styleSheet);
            BasicHTML.BasicDocument basicDocument = new BasicHTML.BasicDocument(styleSheet2, font, color);
            basicDocument.setAsynchronousLoadPriority(Integer.MAX_VALUE);
            basicDocument.setPreservesUnknownTags(false);
            return basicDocument;
        }
        
        @Override
        public ViewFactory getViewFactory() {
            return BasicHTML.basicHTMLViewFactory;
        }
    }
    
    static class BasicHTMLViewFactory extends HTMLEditorKit.HTMLFactory
    {
        @Override
        public View create(Element elem) {
            View create = super.create(elem);
            if ((create instanceof ImageView)) {
                ((ImageView)create).setLoadsSynchronously(true);
                return create;
            }
            return create;
        }
    }
    
    static class BasicDocument extends HTMLDocument
    {
        BasicDocument(StyleSheet styles, Font font, Color color) {
            super(styles);
            this.setPreservesUnknownTags(false);
            this.setFontAndColor(font, color);
        }
        
        private void setFontAndColor(Font font, Color fg) {
            this.getStyleSheet().addRule(SwingUtilities2.displayPropertiesToCSS(font, fg));
        }
    }
    
    static class Renderer extends View
    {
        private int width;
        private View view;
        private ViewFactory factory;
        private JComponent host;
        
        Renderer(JComponent host, ViewFactory factory, View view) {
            super(null);
            this.host = host;
            this.factory = factory;
            (this.view = view).setParent(this);
            this.setSize(this.view.getPreferredSpan(0), this.view.getPreferredSpan(1));
        }
        
        @Override
        public AttributeSet getAttributes() {
            return null;
        }
        
        @Override
        public float getPreferredSpan(int n) {
            if (n == 0) {
                return (float)this.width;
            }
            return this.view.getPreferredSpan(n);
        }
        
        @Override
        public float getMinimumSpan(int axis) {
            return this.view.getMinimumSpan(axis);
        }
        
        @Override
        public float getMaximumSpan(int n) {
            return 2.1474836E9f;
        }
        
        @Override
        public void preferenceChanged(View view, boolean b, boolean b2) {
            this.host.revalidate();
            this.host.repaint();
        }
        
        @Override
        public float getAlignment(int axis) {
            return this.view.getAlignment(axis);
        }
        
        @Override
        public void paint(Graphics graphics, Shape shape) {
            Rectangle bounds = shape.getBounds();
            this.view.setSize((float)bounds.width, (float)bounds.height);
            this.view.paint(graphics, shape);
        }
        
        @Override
        public void setParent(View view) {
            throw new Error("Can't set parent on root view");
        }
        
        @Override
        public int getViewCount() {
            return 1;
        }
        
        @Override
        public View getView(int n) {
            return this.view;
        }
        
        @Override
        public Shape modelToView(int n, Shape shape, Position.Bias bias) throws BadLocationException {
            return this.view.modelToView(n, shape, bias);
        }
        
        @Override
        public Shape modelToView(int p5, Position.Bias b0, int p6, Position.Bias b2, Shape a) throws BadLocationException {
            return this.view.modelToView(p5, b0, p6, b2, a);
        }
        
        @Override
        public int viewToModel(float n, float n2, Shape shape, Position.Bias[] array) {
            return this.view.viewToModel(n, n2, shape, array);
        }
        
        @Override
        public Document getDocument() {
            return this.view.getDocument();
        }
        
        @Override
        public int getStartOffset() {
            return this.view.getStartOffset();
        }
        
        @Override
        public int getEndOffset() {
            return this.view.getEndOffset();
        }
        
        @Override
        public Element getElement() {
            return this.view.getElement();
        }
        
        @Override
        public void setSize(float width, float height) {
            this.width = (int)width;
            this.view.setSize(width, height);
        }
        
        @Override
        public Container getContainer() {
            return this.host;
        }
        
        @Override
        public ViewFactory getViewFactory() {
            return this.factory;
        }
    }
}
